package jp.deadend.noname.dumbgallery

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.preference.PreferenceManager
import com.davemorrissey.labs.subscaleview.ImageSource
import com.davemorrissey.labs.subscaleview.SubsamplingScaleImageView
import jp.deadend.noname.dumbgallery.databinding.FragmentImageViewBinding

class ImageViewFragment : Fragment() {
    private var _binding: FragmentImageViewBinding? = null
    private val binding get() = _binding!!
    private var asset: ImageItem? = null

    fun setAsset(asset: ImageItem) {
        this.asset = asset
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ) : View {
        _binding = FragmentImageViewBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        if (asset == null && savedInstanceState?.containsKey(BUNDLE_ASSET) == true) {
            asset = savedInstanceState.getParcelable(BUNDLE_ASSET)
        }
        asset?.let {
            val (width, height) = it.getImageSize(requireContext().contentResolver)
            val autoRotate = PreferenceManager.getDefaultSharedPreferences(requireContext())
                    .getBoolean(getString(R.string.pref_key_autorotate), false)

            if (autoRotate) {
                val dm = resources.displayMetrics
                if (
                    ((dm.heightPixels > dm.widthPixels) && (width > height)) ||
                    ((dm.widthPixels > dm.heightPixels) && (height > width))
                ) {
                    binding.imageView.orientation = SubsamplingScaleImageView.ORIENTATION_90
                } else {
                    binding.imageView.orientation = SubsamplingScaleImageView.ORIENTATION_0
                }
            }

            if (width < LARGE_IMAGE_THRESHOLD && height < LARGE_IMAGE_THRESHOLD) {
                binding.imageView.setImage(ImageSource.uri(it.uri).tilingDisabled())
            } else {
                binding.imageView.setMaxTileSize(SubsamplingScaleImageView.TILE_SIZE_AUTO - 1)
                binding.imageView.setImage(ImageSource.uri(it.uri))
            }
        }
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        val rootView = view
        if (rootView != null) { outState.putParcelable(BUNDLE_ASSET, asset) }
    }

    fun rotateImage() {
        binding.imageView.orientation = when (binding.imageView.orientation) {
            SubsamplingScaleImageView.ORIENTATION_270 -> SubsamplingScaleImageView.ORIENTATION_180
            SubsamplingScaleImageView.ORIENTATION_180 -> SubsamplingScaleImageView.ORIENTATION_90
            SubsamplingScaleImageView.ORIENTATION_90 -> SubsamplingScaleImageView.ORIENTATION_0
            else -> SubsamplingScaleImageView.ORIENTATION_270
        }
        asset?.let {
            val (width, height) = it.getImageSize(requireContext().contentResolver)
            if (width < LARGE_IMAGE_THRESHOLD && height < LARGE_IMAGE_THRESHOLD) {
                binding.imageView.setImage(ImageSource.uri(it.uri).tilingDisabled())
            } else {
                binding.imageView.setMaxTileSize(SubsamplingScaleImageView.TILE_SIZE_AUTO - 1)
                binding.imageView.setImage(ImageSource.uri(it.uri))
            }
        }
    }

    companion object {
        private const val BUNDLE_ASSET = "asset"
        const val LARGE_IMAGE_THRESHOLD = 2048
        const val POSITION = "POSITION"
        fun getInstance(position: Int): ImageViewFragment {
            return ImageViewFragment().apply {
                arguments = Bundle().also { it.putInt(POSITION, position) }
            }
        }
    }
}