package jp.deadend.noname.dumbgallery

import android.content.Context
import android.os.Bundle
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import android.widget.NumberPicker
import androidx.preference.PreferenceDialogFragmentCompat

class NumberPickerDialogFragment : PreferenceDialogFragmentCompat() {
    private var mValue = NumberPickerPreference.DEFAULT_VALUE
    private lateinit var mNumberPicker: NumberPicker

    override fun onCreateDialogView(context: Context): View {
        mNumberPicker = MyNumberPicker(context)
        mNumberPicker.gravity = Gravity.CENTER
        mNumberPicker.minValue = 1
        mNumberPicker.maxValue = 20
        mNumberPicker.descendantFocusability = NumberPicker.FOCUS_BLOCK_DESCENDANTS

        return mNumberPicker
    }

    override fun onBindDialogView(view: View) {
        super.onBindDialogView(view)

        val pref = preference
        if (pref is NumberPickerPreference) {
            mValue = pref.value
            mNumberPicker.value = mValue
            mNumberPicker.setOnValueChangedListener { _, _, newVal -> mValue = newVal }
        }
    }

    override fun onDialogClosed(positiveResult: Boolean) {
        if (positiveResult) {
            val pref = preference
            if (pref is NumberPickerPreference) { pref.value = mValue }
        }
    }

    private class MyNumberPicker(context: Context) : NumberPicker(context) {
        override fun addView(child: View?, params: ViewGroup.LayoutParams?) {
            super.addView(child, params)
            updateView(child)
        }

        override fun addView(child: View?, index: Int, params: ViewGroup.LayoutParams?) {
            super.addView(child, index, params)
            updateView(child)
        }

        private fun updateView(child: View?) {
            if (child is EditText) { child.setTextSize(TypedValue.COMPLEX_UNIT_SP, 24.0f) }
        }
    }

    companion object {
        fun newInstance(key: String): NumberPickerDialogFragment {
            val fragment = NumberPickerDialogFragment()
            val args = Bundle().apply {
                putString(ARG_KEY, key)
            }
            fragment.arguments = args

            return fragment
        }
    }
}